@@ -1,6 +1,6 @@
 package App::Info::RDBMS::PostgreSQL;
 
-# $Id: PostgreSQL.pm,v 1.1 2002/11/26 19:47:33 theory Exp $
+# $Id: PostgreSQL.pm 11556 2008-07-21 15:15:03Z turnstep $
 
 =head1 NAME
 
@@ -44,9 +44,13 @@ use App::Info::RDBMS;
 use App::Info::Util;
 use vars qw(@ISA $VERSION);
 @ISA = qw(App::Info::RDBMS);
-$VERSION = '0.22';
+$VERSION = '0.45';
+use constant WIN32 => $^O eq 'MSWin32';
 
 my $u = App::Info::Util->new;
+my @EXES = qw(postgres createdb createlang createuser dropdb droplang
+              dropuser initdb pg_dump pg_dumpall pg_restore postmaster
+              vacuumdb psql);
 
 =head1 INTERFACE
 
@@ -59,33 +63,47 @@ my $u = App::Info::Util->new;
 Returns an App::Info::RDBMS::PostgreSQL object. See L<App::Info|App::Info> for
 a complete description of argument parameters.
 
-When it called, C<new()> searches the file system for the F<pg_config>
-application. If found, F<pg_config> will be called by the object methods below
-to gather the data necessary for each. If F<pg_config> cannot be found, then
-PostgreSQL is assumed not to be installed, and each of the object methods will
-return C<undef>.
+When it called, C<new()> searches the file system for an executable named for
+the list returned by C<search_exe_names()>, usually F<pg_config>, in the list
+of directories returned by C<search_bin_dirs()>. If found, F<pg_config> will
+be called by the object methods below to gather the data necessary for
+each. If F<pg_config> cannot be found, then PostgreSQL is assumed not to be
+installed, and each of the object methods will return C<undef>.
 
-App::Info::RDBMS::PostgreSQL searches for F<pg_config> along your path, as
-defined by C<File::Spec-E<gt>path>. Failing that, it searches the following
-directories:
+C<new()> also takes a number of optional parameters in addition to those
+documented for App::Info. These parameters allow you to specify alternate
+names for PostgreSQL executables (other than F<pg_config>, which you specify
+via the C<search_exe_names> parameter). These parameters are:
 
-=over 4
+=over
 
-=item /usr/local/pgsql/bin
+=item search_postgres_names
 
-=item /usr/local/postgres/bin
+=item search_createdb_names
 
-=item /opt/pgsql/bin
+=item search_createlang_names
 
-=item /usr/local/bin
+=item search_createuser_names
 
-=item /usr/local/sbin
+=item search_dropd_names
 
-=item /usr/bin
+=item search_droplang_names
 
-=item /usr/sbin
+=item search_dropuser_names
 
-=item /bin
+=item search_initdb_names
+
+=item search_pg_dump_names
+
+=item search_pg_dumpall_names
+
+=item search_pg_restore_names
+
+=item search_postmaster_names
+
+=item search_psql_names
+
+=item search_vacuumdb_names
 
 =back
 
@@ -115,39 +133,43 @@ sub new {
 
     # Find pg_config.
     $self->info("Looking for pg_config");
-    my @paths = ($u->path,
-      qw(/usr/local/pgsql/bin
-         /usr/local/postgres/bin
-         /opt/pgsql/bin
-         /usr/local/bin
-         /usr/local/sbin
-         /usr/bin
-         /usr/sbin
-         /bin));
 
-    if (my $cfg = $u->first_cat_exe('pg_config', @paths)) {
+    my @paths = $self->search_bin_dirs;
+    my @exes = $self->search_exe_names;
+
+    if (my $cfg = $u->first_cat_exe(\@exes, @paths) and !$ENV{DBDPG_TESTINITDB}) {
         # We found it. Confirm.
         $self->{pg_config} = $self->confirm( key      => 'pg_config',
-                                             prompt   => 'Path to pg_config?',
+                                             prompt   => "Path to pg_config?",
                                              value    => $cfg,
                                              callback => sub { -x },
                                              error    => 'Not an executable');
     } else {
         # Handle an unknown value.
         $self->{pg_config} = $self->unknown( key      => 'pg_config',
-                                             prompt   => 'Path to pg_config?',
+                                             prompt   => "Path to pg_config?",
                                              callback => sub { -x },
                                              error    => 'Not an executable');
     }
 
+    # Set up search defaults.
+    for my $exe (@EXES) {
+        my $attr = "search_$exe\_names";
+        if (exists $self->{$attr}) {
+            $self->{$attr} = [$self->{$attr}] unless ref $self->{$attr} eq 'ARRAY';
+        } else {
+            $self->{$attr} = [];
+        }
+    }
+
     return $self;
 }
 
 # We'll use this code reference as a common way of collecting data.
 my $get_data = sub {
     return unless $_[0]->{pg_config};
-    $_[0]->info("Executing `$_[0]->{pg_config} $_[1]`");
-    my $info = `$_[0]->{pg_config} $_[1]`;
+    $_[0]->info(qq{Executing `"$_[0]->{pg_config}" $_[1]`});
+    my $info = `"$_[0]->{pg_config}" $_[1]`;
     chomp $info;
     return $info;
 };
@@ -229,7 +251,7 @@ my $get_version = sub {
     my $data = $get_data->($self, '--version');
     unless ($data) {
         $self->error("Failed to find PostgreSQL version with ".
-                     "`$self->{pg_config} --version");
+                     "`$self->{pg_config} --version`");
             return;
     }
 
@@ -245,8 +267,13 @@ my $get_version = sub {
     if ($version) {
         my ($x, $y, $z) = $version =~ /(\d+)\.(\d+).(\d+)/;
         if (defined $x and defined $y and defined $z) {
+            # Beta/devel/release candidates are treated as patch level "0"
             @{$self}{qw(version major minor patch)} =
               ($version, $x, $y, $z);
+        } elsif ($version =~ /(\d+)\.(\d+)/) {
+            # New versions, such as "7.4", are treated as patch level "0"
+            @{$self}{qw(version major minor patch)} =
+              ($version, $1, $2, 0);
         } else {
             $self->error("Failed to parse PostgreSQL version parts from " .
                          "string '$version'");
@@ -482,6 +509,83 @@ sub patch_version {
 
 ##############################################################################
 
+=head3 executable
+
+  my $exe = $pg->executable;
+
+Returns the full path to the PostgreSQL server executable, which is named
+F<postgres>.  This method does not use the executable names returned by
+C<search_exe_names()>; those executable names are used to search for
+F<pg_config> only (in C<new()>).
+
+When it called, C<executable()> checks for an executable named F<postgres> in
+the directory returned by C<bin_dir()>.
+
+Note that C<executable()> is simply an alias for C<postgres()>.
+
+B<Events:>
+
+=over 4
+
+=item info
+
+Looking for postgres executable
+
+=item confirm
+
+Path to postgres executable?
+
+=item unknown
+
+Path to postgres executable?
+
+=back
+
+=cut
+
+my $find_exe = sub  {
+    my ($self, $key) = @_;
+    my $exe = $key . (WIN32 ? '.exe' : '');
+    my $meth = "search_$key\_names";
+
+    # Find executable.
+    $self->info("Looking for $key");
+
+    unless ($self->{$key}) {
+        my $bin = $self->bin_dir or return;
+        if (my $exe = $u->first_cat_exe([$self->$meth(), $exe], $bin)) {
+            # We found it. Confirm.
+            $self->{$key} = $self->confirm(
+                key      => $key,
+                prompt   => "Path to $key executable?",
+                value    => $exe,
+                callback => sub { -x },
+                error    => 'Not an executable'
+            );
+        } else {
+            # Handle an unknown value.
+            $self->{$key} = $self->unknown(
+                key      => $key,
+                prompt   => "Path to $key executable?",
+                callback => sub { -x },
+                error    => 'Not an executable'
+            );
+        }
+    }
+
+    return $self->{$key};
+};
+
+for my $exe (@EXES) {
+    no strict 'refs';
+    *{$exe} = sub { shift->$find_exe($exe) };
+    *{"search_$exe\_names"} = sub { @{ shift->{"search_$exe\_names"} } }
+}
+
+*executable = \&postgres;
+
+##############################################################################
+
 =head3 bin_dir
 
   my $bin_dir = $pg->bin_dir;
@@ -673,6 +777,52 @@ sub so_lib_dir {
 
 ##############################################################################
 
+=head3 configure options
+
+  my $configure = $pg->configure;
+
+Returns the options with which the PostgreSQL server was
+configured. App::Info::RDBMS::PostgreSQL gathers the configure data from the
+system call C<`pg_config --configure`>.
+
+B<Events:>
+
+=over 4
+
+=item info
+
+Executing `pg_config --configure`
+
+=item error
+
+Cannot find configure information
+
+=item unknown
+
+Enter PostgreSQL configuration options
+
+=back
+
+=cut
+
+sub configure {
+    my $self = shift;
+    return unless $self->{pg_config};
+    unless (exists $self->{configure} ) {
+        if (my $conf = $get_data->($self, '--configure')) {
+            $self->{configure} = $conf;
+        } else {
+            # Configure can be empty, so just make sure it exists and is
+            # defined. Don't prompt.
+            $self->{configure} = '';
+        }
+    }
+
+    return $self->{configure};
+}
+
+##############################################################################
+
 =head3 home_url
 
   my $home_url = $pg->home_url;
@@ -693,20 +843,205 @@ Returns the PostgreSQL download URL.
 
 =cut
 
-sub download_url { "http://www.ca.postgresql.org/sitess.html" }
+sub download_url { "http://www.postgresql.org/mirrors-ftp.html" }
+
+##############################################################################
+
+=head3 search_exe_names
+
+  my @search_exe_names = $app->search_exe_names;
+
+Returns a list of possible names for F<pg_config> executable. By default, only
+F<pg_config> is returned (or F<pg_config.exe> on Win32).
+
+Note that this method is not used to search for the PostgreSQL server
+executable, only F<pg_config>.
+
+=cut
+
+sub search_exe_names {
+    my $self = shift;
+    my $exe = 'pg_config';
+    $exe .= '.exe' if WIN32;
+    return ($self->SUPER::search_exe_names, $exe);
+}
+
+##############################################################################
+
+=head3 search_bin_dirs
+
+  my @search_bin_dirs = $app->search_bin_dirs;
+
+Returns a list of possible directories in which to search an executable. Used
+by the C<new()> constructor to find an executable to execute and collect
+application info. The found directory will also be returned by the C<bin_dir>
+method.
+
+The list of directories by default consists of the path as defined by
+C<< File::Spec->path >>, as well as the following directories:
+
+=over 4
+
+=item $ENV{POSTGRES_HOME}/bin (if $ENV{POSTGRES_HOME} exists)
+
+=item $ENV{POSTGRES_LIB}/../bin (if $ENV{POSTGRES_LIB} exists)
+
+=item /usr/local/pgsql/bin
+
+=item /usr/local/postgres/bin
+
+=item /opt/pgsql/bin
+
+=item /usr/local/bin
+
+=item /usr/local/sbin
+
+=item /usr/bin
+
+=item /usr/sbin
+
+=item /bin
+
+=item C:\Program Files\PostgreSQL\bin
+
+=back
+
+=cut
+
+sub search_bin_dirs {
+    return shift->SUPER::search_bin_dirs,
+      ( exists $ENV{POSTGRES_HOME}
+          ? ($u->catdir($ENV{POSTGRES_HOME}, "bin"))
+          : ()
+      ),
+      ( exists $ENV{POSTGRES_LIB}
+          ? ($u->catdir($ENV{POSTGRES_LIB}, $u->updir, "bin"))
+          : ()
+      ),
+      $u->path,
+      qw(/usr/local/pgsql/bin
+         /usr/local/postgres/bin
+         /usr/lib/postgresql/bin
+         /opt/pgsql/bin
+         /usr/local/bin
+         /usr/local/sbin
+         /usr/bin
+         /usr/sbin
+         /bin),
+      'C:\Program Files\PostgreSQL\bin';
+}
+
+##############################################################################
+
+=head2 Other Executable Methods
+
+These methods function just like the C<executable()> method, except that they
+return different executables. PostgreSQL comes with a fair number of them; we
+provide these methods to provide a path to a subset of them. Each method, when
+called, checks for an executable in the directory returned by C<bin_dir()>.
+The name of the executable must be one of the names returned by the
+corresponding C<search_*_names> method.
+
+The available executable methods are:
+
+=over
+
+=item postgres
+
+=item createdb
+
+=item createlang
+
+=item createuser
+
+=item dropdb
+
+=item droplang
+
+=item dropuser
+
+=item initdb
+
+=item pg_dump
+
+=item pg_dumpall
+
+=item pg_restore
+
+=item postmaster
+
+=item psql
+
+=item vacuumdb
+
+=back
+
+And the corresponding search names methods are:
+
+=over
+
+=item search_postgres_names
+
+=item search_createdb_names
+
+=item search_createlang_names
+
+=item search_createuser_names
+
+=item search_dropd_names
+
+=item search_droplang_names
+
+=item search_dropuser_names
+
+=item search_initdb_names
+
+=item search_pg_dump_names
+
+=item search_pg_dumpall_names
+
+=item search_pg_restore_names
+
+=item search_postmaster_names
+
+=item search_psql_names
+
+=item search_vacuumdb_names
+
+=back
+
+B<Events:>
+
+=over 4
+
+=item info
+
+Looking for executable
+
+=item confirm
+
+Path to executable?
+
+=item unknown
+
+Path to executable?
+
+=back
+
+=cut
 
 1;
 __END__
 
 =head1 BUGS
 
-Report all bugs via the CPAN Request Tracker at
+Please send bug reports to <bug-app-info@rt.cpan.org> or file them at
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=App-Info>.
 
 =head1 AUTHOR
 
-David Wheeler <L<david@wheeler.net|"david@wheeler.net">> based on code by Sam
-Tregar <L<sam@tregar.com|"sam@tregar.com">>.
+David Wheeler <david@justatheory.com> based on code by Sam Tregar
+<sam@tregar.com>.
 
 =head1 SEE ALSO
 
@@ -722,7 +1057,7 @@ L<http://www.postgresql.org/> is the PostgreSQL home page.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2002, David Wheeler. All Rights Reserved.
+Copyright (c) 2002-2004, David Wheeler. All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
